#!/usr/bin/env python
# -*- coding:utf-8 -*-

from flask import Flask
from flask.ext.sqlalchemy import SQLAlchemy
from flask.ext.script import Manager
from flask.ext.migrate import Migrate, MigrateCommand
try:
    from django.conf import settings
except ImportError:
    import config as settings

app = Flask(__name__)
app.config['SQLALCHEMY_DATABASE_URI'] = settings.ANTI_SQLALCHEMY_DATABASE_URI

db = SQLAlchemy(app)
migrate = Migrate(app, db)

manager = Manager(app)
manager.add_command('db', MigrateCommand)

class Key(db.Model):
    id = db.Column(db.Integer, primary_key = True)
    key = db.Column(db.String(255))

class Host(db.Model):
    id = db.Column(db.Integer, primary_key = True)
    host = db.Column(db.String(255))

class Url(db.Model):
    id = db.Column(db.Integer, primary_key = True)
    url = db.Column(db.String(1023))

class Pos(db.Model):
    id = db.Column(db.BigInteger, primary_key = True)
    date = db.Column(db.Date)
    key_id = db.Column(db.Integer, db.ForeignKey('key.id'))
    host_id = db.Column(db.Integer, db.ForeignKey('host.id'))
    url_id = db.Column(db.Integer, db.ForeignKey('url.id'))
    pos = db.Column(db.SmallInteger)
    search = db.Column(db.SmallInteger)
    lr = db.Column(db.SmallInteger, default = 213)

class SeoData(db.Model):
    id = db.Column(db.Integer, primary_key = True)
    date = db.Column(db.Date)
    host_id = db.Column(db.Integer, db.ForeignKey('host.id'))
    tic = db.Column(db.SmallInteger)
    pr = db.Column(db.SmallInteger)
    yaca = db.Column(db.Boolean)
    indexed_google = db.Column(db.Integer)
    indexed_google_clear = db.Column(db.Integer)
    indexed_yandex = db.Column(db.Integer)
    donors = db.Column(db.Integer)
    backlinks = db.Column(db.Integer)
    wm_tic = db.Column(db.Integer)
    wm_indexed = db.Column(db.Integer)

class Visits(db.Model):
    id = db.Column(db.Integer, primary_key = True)
    date = db.Column(db.Date)
    host_id = db.Column(db.Integer, db.ForeignKey('host.id'))
    av = db.Column(db.Integer)
    rv = db.Column(db.Integer)
    sv = db.Column(db.Integer)

class Token(db.Model):
    id = db.Column(db.Integer, primary_key = True)
    token = db.Column(db.String(255))

if __name__ == '__main__':
    manager.run()



# -*- coding:utf-8 -*-

import datetime
import requests

from bs4 import BeautifulSoup

from .utils import get_normal_url_decode, get_or_create, transaction
from .anti_config import db, Host, SeoData, Token


class Wrapper(object):
    def __init__(self, site, user, passwd, client_secret, client_id):
        self.site = get_normal_url_decode(site)
        self.user = user
        self.passwd = passwd
        self.client_secret = client_secret
        self.client_id = client_id
        with transaction():
            self.host = get_or_create(db.session, Host, host=self.site)
        self.tkn = Token.query.first()
        if self.tkn:
            self.access_token = self.tkn.token
        self.url = self.get_url()

    def auth(self):
        """ Авторизация """
        self.user = self.user
        self.passwd = self.passwd
        self.client_secret = self.client_secret
        self.client_id = self.client_id

        url = 'https://oauth.yandex.ru/authorize?response_type=token&client_id=%s' % self.client_id
        resp = requests.get(url, timeout=120)
        soup = BeautifulSoup(resp.text)
        data = {'login': self.user, 'passwd': self.passwd}
        for item in soup.findAll('input', {'type': 'hidden'}):
            data[item['name']] = item['value']
        url = soup.find('form')['action'] if 'action' in soup.find('form') else resp.url

        resp = requests.post(url, data, timeout=120)
        soup = BeautifulSoup(resp.text)
        if soup.find('button', {'class': ' nb-button _nb-action-button _init'}):
            url = 'https://oauth.yandex.ru' + soup.find('form')['action']
            for item in soup.findAll('input', {'type': 'hidden'}):
                data[item['name']] = item['value']
            resp = requests.post(url, data, timeout=120)
        self.access_token = resp.url[resp.url.find('#'):].replace('#', '').split('&')[0].split('=')[1]
        if not self.tkn:
            self.tkn = Token(token=self.access_token)
            db.session.add(self.tkn)
        else:
            self.tkn.token = self.access_token

        with transaction():
            db.session.commit()

    def get_url(self):
        """ Получение адреса для api"""
        if not hasattr(self, 'access_token'):
            self.auth()
        self.headers = {'Authorization': 'OAuth %s' % self.access_token}
        resp = requests.get('https://webmaster.yandex.ru/api/v2/hosts', headers=self.headers, timeout=120)
        soup = BeautifulSoup(resp.text)
        hosts = soup.findAll('host')
        for host in hosts:
            name = host.find('name').text.lower()
            if name in self.site or self.site in name:
                url = host['href']
                return url
        return None


class Webmaster(Wrapper):

    def commit(self):
        with transaction():
            if hasattr(self, 'history_tic'):
                for value in self.history_tic:
                    date = datetime.datetime.strptime(value['date'], '%Y-%m-%d')
                    res = get_or_create(db.session, SeoData, date=date, host_id=self.host.id)
                    res.wm_tic = int(value['value'])

            if hasattr(self, 'indexed'):
                res = get_or_create(db.session, SeoData, date=datetime.date.today(), host_id=self.host.id)
                res.wm_indexed = self.indexed

            if hasattr(self, 'history_indexed'):
                for value in self.history_indexed:
                    date = datetime.datetime.strptime(value['date'], '%Y-%m-%d')
                    res = get_or_create(db.session, SeoData, date=date, host_id=self.host.id)
                    res.wm_indexed = value['value']
            db.session.commit()

    def get_tic(self):
        if self.url:
            url = self.url + '/history/tic'
            rsst = requests.get(url, headers=self.headers, timeout=120)
            soup = BeautifulSoup(rsst.text)
            self.history_tic = [{'date': item['date'], 'value': item.text} for item in soup.findAll('value')]
            return self.history_tic
        return None

    def get_indexed(self):
        if self.url:
            url = self.url + '/indexed/'
            rsst = requests.get(url, headers=self.headers, timeout=120)
            soup = BeautifulSoup(rsst.text)
            self.indexed = 0
            if soup.find('index-count'):
                self.indexed = soup.find('index-count').text
            return self.indexed
        return None

    def get_history_indexed(self):
        if self.url:
            url = self.url + '/history/indexed-urls'
            rsst = requests.get(url, headers=self.headers, timeout=120)
            soup = BeautifulSoup(rsst.text)
            self.history_indexed = [{'date': item['date'], 'value': item.text} for item in soup.findAll('value')]
            return self.history_indexed
        return None

    def get_original_texts(self):
        # WTF к яндексу
        if self.url:
            url = self.url + '/original-texts/'
            rsst = requests.get(url, headers=self.headers, timeout=120)
            soup = BeautifulSoup(rsst.text)
            texts = soup.findAll('original-text')
            return texts
        return None
